# ipdigitialcommon.tcl

#    common part of IP-DIGITAL48   and   IP-Dual PI/T
# 

set this_manid $greenspring_id
set this_modid $dpit_id

set PITIDs [list X Y]
set PIT68230 [first $PITIDs]
set PITBase([first $PITIDs]) 0x00
set PITBase([second $PITIDs]) 0x40

set Rnames {PGCR PSRR PADDR PBDDR PCDDR PIVR PACR PBCR PADR PBDR PCDR PSR TCR TIVR CPRH CPRM CPRL TSR}
foreach r $Rnames {set [set r]bits 0}
set Rlabels(PGCR)   "Port General Control Register"
set Rlabels(PSRR)   "Port Service Request Register"
set Rlabels(PADDR)  "Port A Data Direction Register"
set Rlabels(PBDDR)  "Port B Data Direction Register"
set Rlabels(PCDDR)  "Port C Data Direction Register"
set Rlabels(PIVR)   "Port Interrupt Vector Register"
set Rlabels(PACR)   "Port A Control Register"
set Rlabels(PBCR)   "Port B Control Register"
set Rlabels(PADR)   "Port A Data Register"
set Rlabels(PBDR)   "Port B Data Register"
set Rlabels(PCDR)   "Port C Data Register"
set Rlabels(PSR)    "Port Status Register"
set Rlabels(TCR)    "Timer Control Register"
set Rlabels(TIVR)   "Timer Interrupt Vector Register"
set Rlabels(CPRH)   "Counter Preload Register (High)"
set Rlabels(CPRM)   "Counter Preload Register (Med)"
set Rlabels(CPRL)   "Counter Preload Register (Low)"
set Rlabels(TSR)    "Timer Status Register"


proc warn-wrongIP {} {
   midas-warning "This IP pack is not a Dual PI/T"
}


proc define-module {} {
  global resource mname VIPC616BaseAddress IPslot
  global passed this_manid this_modid PITIDs PITBase

  set-footer "deleting all IP-DPIT registers"
  EG "register :pattern '/$resource/$mname.*' !delete"

#     ID
      define-register xIP ManID        "-a $VIPC616BaseAddress -s $IPslot -o 0x09 -d id -p ro -w 8 -zx"
      define-register xIP ModID        "-a $VIPC616BaseAddress -s $IPslot -o 0x0b -d id -p ro -w 8 -zx"

     set manid [read-register ManID]
     update-widget MANID $manid

     set modid [read-register ModID]
     update-widget MODID $modid

     if {$manid == $this_manid  && $modid == $this_modid} then {
        set passed 1
     } else { set passed 0 }
     update-widget PASSED $passed

     if {$passed} then {

#     Control & Data Registers
#	68230 PIT X

      foreach i $PITIDs {
         set RegBase $PITBase($i)
         define-register xIP PGCR_$i   "-a $VIPC616BaseAddress -s $IPslot -o [expr $RegBase + 0x01] -d io -p rw -w 8 -zx"
         define-register xIP PSRR_$i   "-a $VIPC616BaseAddress -s $IPslot -o [expr $RegBase + 0x03] -d io -p rw -w 8 -zx"
         define-register xIP PADDR_$i  "-a $VIPC616BaseAddress -s $IPslot -o [expr $RegBase + 0x05] -d io -p rw -w 8 -zx"
         define-register xIP PBDDR_$i  "-a $VIPC616BaseAddress -s $IPslot -o [expr $RegBase + 0x07] -d io -p rw -w 8 -zx"
         define-register xIP PCDDR_$i  "-a $VIPC616BaseAddress -s $IPslot -o [expr $RegBase + 0x09] -d io -p rw -w 8 -zx"
         define-register xIP PIVR_$i   "-a $VIPC616BaseAddress -s $IPslot -o [expr $RegBase + 0x0B] -d io -p rw -w 8 -zx"
         define-register xIP PACR_$i   "-a $VIPC616BaseAddress -s $IPslot -o [expr $RegBase + 0x0D] -d io -p rw -w 8 -zx"
         define-register xIP PBCR_$i   "-a $VIPC616BaseAddress -s $IPslot -o [expr $RegBase + 0x0F] -d io -p rw -w 8 -zx"
         define-register xIP PADR_$i   "-a $VIPC616BaseAddress -s $IPslot -o [expr $RegBase + 0x11] -d io -p rw -w 8 -zx"
         define-register xIP PBDR_$i   "-a $VIPC616BaseAddress -s $IPslot -o [expr $RegBase + 0x13] -d io -p rw -w 8 -zx"
         define-register xIP PAAR_$i   "-a $VIPC616BaseAddress -s $IPslot -o [expr $RegBase + 0x15] -d io -p ro -w 8 -zx"
         define-register xIP PBAR_$i   "-a $VIPC616BaseAddress -s $IPslot -o [expr $RegBase + 0x17] -d io -p ro -w 8 -zx"
         define-register xIP PCDR_$i   "-a $VIPC616BaseAddress -s $IPslot -o [expr $RegBase + 0x19] -d io -p rw -w 8 -zx"
         define-register xIP PSR_$i    "-a $VIPC616BaseAddress -s $IPslot -o [expr $RegBase + 0x1B] -d io -p rw -w 8 -zx"
         define-register xIP TCR_$i    "-a $VIPC616BaseAddress -s $IPslot -o [expr $RegBase + 0x21] -d io -p rw -w 8 -zx"
         define-register xIP TIVR_$i   "-a $VIPC616BaseAddress -s $IPslot -o [expr $RegBase + 0x23] -d io -p rw -w 8 -zx"
         define-register xIP CPRH_$i   "-a $VIPC616BaseAddress -s $IPslot -o [expr $RegBase + 0x27] -d io -p rw -w 8 -zx"
         define-register xIP CPRM_$i   "-a $VIPC616BaseAddress -s $IPslot -o [expr $RegBase + 0x29] -d io -p rw -w 8 -zx"
         define-register xIP CPRL_$i   "-a $VIPC616BaseAddress -s $IPslot -o [expr $RegBase + 0x2B] -d io -p rw -w 8 -zx"
         define-register xIP CNTRH_$i  "-a $VIPC616BaseAddress -s $IPslot -o [expr $RegBase + 0x2F] -d io -p ro -w 8 -zx"
         define-register xIP CNTRM_$i  "-a $VIPC616BaseAddress -s $IPslot -o [expr $RegBase + 0x31] -d io -p ro -w 8 -zx"
         define-register xIP CNTRL_$i  "-a $VIPC616BaseAddress -s $IPslot -o [expr $RegBase + 0x33] -d io -p ro -w 8 -zx"
         define-register xIP TSR_$i    "-a $VIPC616BaseAddress -s $IPslot -o [expr $RegBase + 0x35] -d io -p rw -w 8 -zx"
      }

#    Update IP details

     click-redisplay
   } else {warn-wrongIP}
}

proc update-widget {widget value} {
   :modify $widget :value $value
}

proc reverse-bits b {
   set bits 0
   for {set i 0} {$i < 8} {incr i} {
      set bits [expr $bits | ((($b >> $i) & 1) << (7 - $i))]
   }
   set bits
}

proc click-redisplay {} {
    global passed Rnames

    if {!$passed} then {warn-wrongIP; return}

    foreach r $Rnames {click-Rreg $r}
    click-Rcount
}

proc click-bits {w v} {
    global passed

    if {!$passed} then {warn-wrongIP; return}

    set r [first [split $w -]]
    global [set r]bits
    set [set r]bits [reverse-bits $v]
}

proc click-Rreg {w} {
    global passed PIT68230

    if {!$passed} then {warn-wrongIP; return}

    set r [first [split $w -]]
    global [set r]bits
    set [set r]bits [expr [read-register [set r]_[set PIT68230]]]

    update-widget [set r]-BITS [reverse-bits [set [set r]bits]]
}

proc click-Wreg {w} {
    global passed PIT68230

    if {!$passed} then {warn-wrongIP; return}

    set r [first [split $w -]]
    global [set r]bits
    write-register [set r]_[set PIT68230] [set [set r]bits]
}

proc click-Hreg {w} {
    upvar #0 :help help

    set r [first [split $w -]]
    if {![info exists help($r)]} then {return}

    midas-help $r  ""  $help($r)
}


proc click-Rcount {} {
    global passed PIT68230

    if {!$passed} then {warn-wrongIP; return}

    set high [read-register CNTRH_[set PIT68230]]
    set med  [read-register CNTRM_[set PIT68230]]
    set low  [read-register CNTRL_[set PIT68230]]

    update-widget CNTR "$high   $med   $low"
}

proc click-setup {args} {

#    set all port pins as outputs 

    write-register PADDR_X 255
    write-register PADDR_Y 255

    write-register PBDDR_X 255
    write-register PBDDR_Y 255

    write-register PCDDR_X 255
    write-register PCDDR_Y 255

#    set all data bits => 0

    write-register PADR_X 0
    write-register PADR_Y 0

    write-register PBDR_X 0
    write-register PBDR_Y 0

    write-register PCDR_X 0
    write-register PCDR_Y 0

#    set for simple bit i/o    mode 0 submode 1X

    write-register PGCR_X 0
    write-register PGCR_Y 0

    write-register PACR_X 0x80
    write-register PBCR_X 0x80

    write-register PACR_Y 0x80
    write-register PBCR_Y 0x80

    click-redisplay
}

proc click arg {
  global debug
  global VIPC616BaseAddress IPslot PIT68230 PITIDs

   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      IPBASEADDR   {
          set VIPC616BaseAddress $v
          define-module
      }
      IPSLOTADDR {
          set IPslot $v
          define-module
      }
      PIT {
          set PIT68230 [lindex $PITIDs $v]
          click-redisplay
      }

      *-BITS     {click-bits $w $v}
      *-R        {click-Rreg    $w}
      *-W        {click-Wreg    $w}
      *-H        {click-Hreg    $w}

      CNTRR      {click-Rcount}

      default   {puts stdout $arg}
   }
}

proc make-frame {} {
  global env VIPC616BaseAddress IPslot PIT68230 PITIDs Rlabels Rnames

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/charissa" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 20

      :text IPBASEADDR   :label "IP Carrier Base Address:" :value $VIPC616BaseAddress :w 12
      :choice IPSLOTADDR :label "IP Pack slot" :strings [list A B C D E F] :value $IPslot
      :next-row T

      :text MANID        :label "Man ID"  :w 4
      :text MODID        :label "Mod ID"  :w 4
      :checkbox PASSED       :label "IP check passed"
      :next-row T

      :choice PIT :label "PIT (68230)" :strings $PITIDs :value [lsearch $PITIDs $PIT68230]
      :next-row T

   :panel CLICK :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 15

       foreach r $Rnames {
          :choice $r-BITS :label "$Rlabels($r) ($r)" :strings [list 7 6 5 4 3 2 1 0] :exclusive F :valuex 40c
          :button $r-W    :label Write :w 5
          :button $r-R    :label Read  :w 5
          :button $r-H    :label "?"
          :next-row T
      }
      :text CNTR :label "Count Register(s) (CNTRH-L)" :valuex 40c :deaf T
      :button CNTRR :label Read
   
   :panel TEST  :layout T :padx 10 :pady 10
      :layout H; :col-gap 15
      :button REDISPLAY :label "Redisplay"              :command click-redisplay
      :button SETUP     :label "Setup"                  :command click-setup
      :button DEFINE    :label "Define IP registers" :command define-module
   
   :show T
}


proc start args {
   global started
   if {$started == 0} then {
      make-frame
      set started 1
      define-module
#      click-setup
   } else {open-frame}
}
