# ipquadrature.tcl---IP-Quadrature IP pack diagnostic frame
# 

set debug 0
set started 0

set testmode 0
set monitor 0

set frame ipquadrature
set frametitle "IP-Quadrature Diagnostics Frame"
set icontitle "ipquadrature"

if {[info exists env(VmeServer)]} then {set resource $env(VmeServer)} \
    else {set resource VME}

set mname "IPQUADRATURE"

set VIPC616BaseAddress 0xffffc000
set IPslot 0

set channel 1

loadsource ipcommon


proc define-module {} {
  global resource mname IPslot VIPC616BaseAddress
  set-footer "deleting all IP-Quadrature registers"
  EG "register :pattern '/$resource/$mname.*' !delete"

#     ID
      define-register xIP ManID        "-a $VIPC616BaseAddress -s $IPslot -o 0x09 -d id -p ro -w 8 -zx"
      define-register xIP ModID        "-a $VIPC616BaseAddress -s $IPslot -o 0x0b -d id -p ro -w 8 -zx"

      define-register xIP CSR          "-a $VIPC616BaseAddress -s $IPslot -o 0x31 -d io -p rw -w 8 -zx"
#      define-register xIP VR           "-a $VIPC616BaseAddress -s $IPslot -o 0x35 -d io -p rw -w 8 -zx"

#     Counter 1
      define-register xIP Ch1.Counter  "-a $VIPC616BaseAddress -s $IPslot -o 0x01 -d io -p ro -w 8 -zx"
      define-register xIP Ch1.PCR      "-a $VIPC616BaseAddress -s $IPslot -o 0x01 -d io -p wo -w 8 -zx"
      define-register xIP Ch1.Status   "-a $VIPC616BaseAddress -s $IPslot -o 0x03 -d io -p ro -w 8 -zx"
      define-register xIP Ch1.Control  "-a $VIPC616BaseAddress -s $IPslot -o 0x03 -d io -p wo -w 8 -zx"
      define-register xIP Ch1.ICR      "-a $VIPC616BaseAddress -s $IPslot -o 0x03 -d io -p wo -w 8 -zx"
      define-register xIP Ch1.OCR      "-a $VIPC616BaseAddress -s $IPslot -o 0x03 -d io -p wo -w 8 -zx"
      define-register xIP Ch1.QR       "-a $VIPC616BaseAddress -s $IPslot -o 0x03 -d io -p wo -w 8 -zx"
      define-register xIP Ch1.CHCR     "-a $VIPC616BaseAddress -s $IPslot -o 0x21 -d io -p rw -w 8 -zx"

#     Counter 2
      define-register xIP Ch2.Counter  "-a $VIPC616BaseAddress -s $IPslot -o 0x05 -d io -p ro -w 8 -zx"
      define-register xIP Ch2.PCR      "-a $VIPC616BaseAddress -s $IPslot -o 0x05 -d io -p wo -w 8 -zx"
      define-register xIP Ch2.Status   "-a $VIPC616BaseAddress -s $IPslot -o 0x07 -d io -p ro -w 8 -zx"
      define-register xIP Ch2.Control  "-a $VIPC616BaseAddress -s $IPslot -o 0x07 -d io -p wo -w 8 -zx"
      define-register xIP Ch2.ICR      "-a $VIPC616BaseAddress -s $IPslot -o 0x07 -d io -p wo -w 8 -zx"
      define-register xIP Ch2.OCR      "-a $VIPC616BaseAddress -s $IPslot -o 0x07 -d io -p wo -w 8 -zx"
      define-register xIP Ch2.QR       "-a $VIPC616BaseAddress -s $IPslot -o 0x07 -d io -p wo -w 8 -zx"
      define-register xIP Ch2.CHCR     "-a $VIPC616BaseAddress -s $IPslot -o 0x25 -d io -p rw -w 8 -zx"

#     Counters 3
      define-register xIP Ch3.Counter  "-a $VIPC616BaseAddress -s $IPslot -o 0x09 -d io -p ro -w 8 -zx"
      define-register xIP Ch3.PCR      "-a $VIPC616BaseAddress -s $IPslot -o 0x09 -d io -p wo -w 8 -zx"
      define-register xIP Ch3.Status   "-a $VIPC616BaseAddress -s $IPslot -o 0x0b -d io -p ro -w 8 -zx"
      define-register xIP Ch3.Control  "-a $VIPC616BaseAddress -s $IPslot -o 0x0b -d io -p wo -w 8 -zx"
      define-register xIP Ch3.ICR      "-a $VIPC616BaseAddress -s $IPslot -o 0x0b -d io -p wo -w 8 -zx"
      define-register xIP Ch3.OCR      "-a $VIPC616BaseAddress -s $IPslot -o 0x0b -d io -p wo -w 8 -zx"
      define-register xIP Ch3.QR       "-a $VIPC616BaseAddress -s $IPslot -o 0x0b -d io -p wo -w 8 -zx"
      define-register xIP Ch3.CHCR     "-a $VIPC616BaseAddress -s $IPslot -o 0x29 -d io -p rw -w 8 -zx"

#     Counters 4
      define-register xIP Ch4.Counter  "-a $VIPC616BaseAddress -s $IPslot -o 0x0d -d io -p ro -w 8 -zx"
      define-register xIP Ch4.PCR      "-a $VIPC616BaseAddress -s $IPslot -o 0x0d -d io -p wo -w 8 -zx"
      define-register xIP Ch4.Status   "-a $VIPC616BaseAddress -s $IPslot -o 0x0f -d io -p ro -w 8 -zx"
      define-register xIP Ch4.Control  "-a $VIPC616BaseAddress -s $IPslot -o 0x0f -d io -p wo -w 8 -zx"
      define-register xIP Ch4.ICR      "-a $VIPC616BaseAddress -s $IPslot -o 0x0f -d io -p wo -w 8 -zx"
      define-register xIP Ch4.OCR      "-a $VIPC616BaseAddress -s $IPslot -o 0x0f -d io -p wo -w 8 -zx"
      define-register xIP Ch4.QR       "-a $VIPC616BaseAddress -s $IPslot -o 0x0f -d io -p wo -w 8 -zx"
      define-register xIP Ch4.CHCR     "-a $VIPC616BaseAddress -s $IPslot -o 0x2d -d io -p rw -w 8 -zx"

     click-redisplay
}


proc read-SR {c} {return [expr [read-register Ch$c.Status] & 0x1f]}

proc write-CR {c v} {write-register Ch$c.Control [expr $v & 0x3f]}

proc write-ICR {c v} {write-register Ch$c.ICR [expr ($v & 0x3f) | 0x40]}

proc write-OCR {c v} {write-register Ch$c.OCR [expr ($v & 0x3f) | 0x80]}

proc write-QR {c v} {write-register Ch$c.QR [expr ($v & 0x3f) | 0xc0]}

proc write-CHCR {c v} {write-register Ch$c.CHCR [expr ($v & 0x9f) | 0x20]}

proc read-CHCR {c} {return [expr [read-register Ch$c.CHCR] & 0xdf]}

proc read-CSR {} {return [expr [read-register CSR] & 0x03]}

proc write-CSR {v} {write-register CSR [expr ($v & 0x03) | 0xf8]}



#    read the channel counter register
proc read-counter {c} {

#    write bits [1..0] of the counter control register to 1 bits. 
#    this resets the byte sequence pointer and transfers the counter contents to the Output Latch
     write-CR $c 0x03

#    read the 3 bytes of the Output Latch
     set b0 [read-register Ch$c.Counter]
     set b1 [read-register Ch$c.Counter]
     set b2 [read-register Ch$c.Counter]

     return [expr ((($b2 << 16) + ($b1 << 8) + $b0) << 8) >> 8]
}

proc write-counter {c v} {

     set b0 [expr $v & 255]
     set b1 [expr ($v >> 8) & 255]
     set b2 [expr ($v >> 16) & 255]

#    set bit 0 of the counter control register to 1
#    this resets the byte sequence pointer
     write-CR $c 0x01

#    write the 3 bytes of the PCR
     write-register Ch$c.PCR $b0
     write-register Ch$c.PCR $b1
     write-register Ch$c.PCR $b2

#    set bit 3 of the counter control register to 1
#    this transfers the contents of the PCR to the counter
     write-CR $c 0x08
}

proc zero-counter {c} {write-CR $c 0x04}

proc reset-counter {c} {

#    set bit 5 of the counter control register to 1
#    this resets the Counter
     write-CR $c 0x20
}


proc update-widget {widget value} {
   :modify $widget :value $value
}

proc click-redisplay {} {
    global channel

     set manid [read-register ManID]
     update-widget MANID $manid

     set modid [read-register ModID]
     update-widget MODID $modid

     if {$manid == 0xf0 && $modid == 0x41} then {set passed 1} else {set passed 0}
     update-widget PASSED $passed

     update-widget CSR [format 0x%x [read-CSR]]

     update-widget COUNTER [read-counter $channel]
     update-widget SR [format 0x%x [read-SR $channel]]

     update-widget CHCR [format 0x%x [read-CHCR $channel]]
}

proc click-reset {args} {}

proc click arg {
   global debug
  global VIPC616BaseAddress IPslot
  global channel

   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      IPBASEADDR   {
          set VIPC616BaseAddress $v
          define-module
      }
      IPSLOTADDR {
          set IPslot $v
          define-module
      }

      CHANNEL  {set channel [expr $v + 1]; click-redisplay}
      COUNTER  {write-counter $channel $v}
      UPDATE   {update-widget COUNTER [read-counter $channel]}

      ZERO     {zero-counter $channel}
      RESET    {reset-counter $channel}

      CSR      {write-CSR $v}
      CHCR     {write-CHCR $channel $v}

      CR        {write-CR $channel $v}
      ICR       {write-ICR $channel $v}
      OCR       {write-OCR $channel $v}
      QR        {write-QR $channel $v}
 
      default   {insert-debug "$arg"}
   }
}


proc make-frame {} {
  global env VIPC616BaseAddress IPslot
  global channel

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/charissa" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 5

      :text IPBASEADDR   :label "IP Carrier Base Address:" :value $VIPC616BaseAddress :w 10
      :choice IPSLOTADDR :label "IP Pack slot" :strings [list A B C D E F]         :value $IPslot
      :next-row T; :col-gap 20
      :text MANID        :label "Man ID"  :w 4
      :text MODID        :label "Mod ID"  :w 4
      :checkbox PASSED       :label "IP check passed"

      :next-row T
      :text CSR :label "Control and Status Register" :w 6

   :panel CLICK :layout T :padx 10
      :layout H; :row-gap 5; :col-gap 15

      :choice CHANNEL :label "Channel"  :strings [list 1 2 3 4] :value [expr $channel -1]
      :hskip 50
      :text COUNTER  :label "Counter" :w 8
      :button UPDATE 

      :next-row T
      :text CHCR     :label "Channel Configuration Register" :w 6 :valuex 26c
      :text SR       :label "Status Register"        :deaf T :w 6 :valuex 20c

      :next-row T
      :text CR       :label "Counter Control Register" :w 6 :valuex 26c
      :text ICR      :label "Input Control Register"   :w 6 :valuex 20c

      :next-row T
      :text OCR      :label "Output Control Register"  :w 6 :valuex 26c
      :text QR       :label "Quadrature Register"      :w 6 :valuex 20c

      :next-row T
      :button RESET :label "Reset Channel" 
      :button ZERO  :label "Zero Counter"
   
   :panel TEST  :layout T :padx 10 :pady 10
      :layout H; :col-gap 15; :row-gap 10
      :button REDISPLAY :label "Redisplay"              :command click-redisplay
      :button DEFINE    :label "Define IP-Quadrature registers" :command define-module
   
   :show T
}


proc start args {
   global started
   if {$started == 0} then {
      make-frame
      set started 1
#      define-module
#      click-reset
   } else {open-frame}
}

proc Start {s} {
    global IPslot started

    set IPslot $s
    set started 0
    start
}
