# STR721.tcl---Struck 721 Scaler module control frame


set frame STR721
set frametitle "Struck 721 Scaler Setup & Monitor"
set icontitle "STR721"

set modulename str721
set moduletype  "Struck STR721"

loadsource common.vme

get-module-configuration     ;# needed early

set NUM 16
set maxcounter [expr $NUM * $maxmodule]

set adc  0

set rates 0
set rateperiod 10
set elapseperiod 10400

for {set c 0} {$c < $maxcounter} {incr c} {set data($c) 0; set data1($c) 0; set data2($c) 0; set rate($c) 0}
set rateid 0



proc click_rates {v} {
   global rates rateperiod rateid data data1 data2 maxcounter
   set rates $v
   :modify RATES :value $rates
   if {$rates == 0} then {
       after cancel $rateid
       set rateid 0
       display_rates
   } else {
      if {$rateid != 0} then {return}
      display_counters
      for {set c 0} {$c < $maxcounter} {incr c} {set data2($c) $data($c)}
      set rateid [after [expr $rateperiod * 1000] rates_update]
   }
}

proc rates_update {} {
   global rates rateperiod rateid data data1 data2 maxcounter
   if {$rates == 0} then {return}

   if {[:query_Mapped .]} then {display_counters}
   for {set c 0} {$c < $maxcounter} {incr c} {set data1($c) $data($c)}
   if {[:query_Mapped .]} then {display_rates}
   for {set c 0} {$c < $maxcounter} {incr c} {set data2($c) $data1($c)}
   set rateid [after [expr $rateperiod * 1000] rates_update]
}


proc click_clear {} {
   global modulename

#  clear all counters
#  LOAD all counters in each chip  (the load register contains 0)
     write-vme-registers $modulename#?.c?.control 0xff5f
}

proc click_arm {} {
   global modulename

#  arm all counters
     write-vme-registers $modulename#?.c?.control 0xff2f
}

proc click_disarm {} {
   global modulename

#  disarm all counters
     write-vme-registers $modulename#?.c?.control 0xffcf
}

set str721_counter_mode(1) 0x0128
set str721_counter_mode(2) 0x0028
set str721_counter_mode(3) 0x0328
set str721_counter_mode(4) 0x0028
set str721_counter_mode(5) 0x0d28

proc click_reset {} {
   global modulename  str721_counter_mode

#     reset all the AMD9513 chips on the module
#  master reset
         write-vme-registers $modulename#?.c?.control 0xffff
#  LOAD all counters
         write-vme-registers $modulename#?.c?.control 0xff5f
#  set data pointer register to a valid code
         write-vme-registers $modulename#?.c?.control 0xff01
#  set MM13 - enter 16bit bus mode          
         write-vme-registers $modulename#?.c?.control 0xffef
#  set MM14 - disable data pointer sequencing         
         write-vme-registers $modulename#?.c?.control 0xffe8

#    configure the 5 counters on each chip to act as 2 32-bit counters  (the 5th counter is not in fact used)
        for {set j 1} {$j <= 5} {incr j} {
#         set the data pointer register to the mode register for counter j
             write-vme-registers $modulename#?.c?.control [expr 0xff00 + $j]
#         set the mode register
             write-vme-registers $modulename#?.c?.data $str721_counter_mode($j)
#         set the data pointer register to the load register for counter j
             write-vme-registers $modulename#?.c?.control [expr 0xff08 + $j]
#         set the load register to 0
             write-vme-registers $modulename#?.c?.data 0
#         set the data pointer register to the hold register for counter j
             write-vme-registers $modulename#?.c?.control [expr 0xff10 + $j]
        }
}

proc display_counters {} {
   global modulelist module

   foreach module $modulelist {display_modulecounters}
}

proc display_modulecounters {} {
   global modulename module data EGreport

   set EGreport -1
   set-busy

#    save all counters in the Hold register
   write-vme-registers $modulename#$module.c?.control 0xffbf

   set c [expr ($module - 1) * 16]
   for {set i 0} {$i < 8} {incr i} {
#    select hold register low word (counter 1)
       write-vme-register $modulename#$module.c$i.control 0xff11
#    read it
       set datal [read-vme-register $modulename#$module.c$i.data]
#    select hold register high word (counter 2)
       write-vme-register $modulename#$module.c$i.control 0xff12
#    read it
       set datah [read-vme-register $modulename#$module.c$i.data]

       set data($c) [expr $datal + ($datah << 16)]
       :modify C[set c]DATA  :value $data($c)
       incr c

#    select hold register low word (counter 3)
       write-vme-register $modulename#$module.c$i.control 0xff13
#    read it
       set datal [read-vme-register $modulename#$module.c$i.data]
#    select hold register high word (counter 4)
       write-vme-register $modulename#$module.c$i.control 0xff14
#    read it
       set datah [read-vme-register $modulename#$module.c$i.data]

       set data($c) [expr $datal + ($datah << 16)]
       :modify C[set c]DATA  :value $data($c)
       incr c
   }
   clear-busy
}

proc display_rates {} {
   global rate data1 data2 rates elapseperiod maxcounter

   clear-footer
   for {set c 0} {$c < $maxcounter} {incr c} {
       if {$rates == 0} then {
          set data 0
       } else {
          set data [expr (($data1($c) - $data2($c)) * 1000) / $elapseperiod]
       }
       set rate($c) $data
       :modify C[set c]RATE  :value $rate($c)
   }
   set-footer "Rates last updated [date]"
}


proc click-redisplay {} {display_counters}

proc click-select-module {v} {
    global module
    set module $v
    :modify MODULE-NUM :value $module
    click_rates 0
    display_counters
}
    

proc click arg {
   global debug actionlist
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {click-quit; return}
   switch $w {

      MODULE-SEL      -
      MODULE-NUM      {click-select-module $v}

      RATES  {click_rates $v}

      ACTIONS  {
         set action [lindex $actionlist $i]
         eval [second $action]
      }

      default           {
           set p "click_[lowercase $w]"
           if {[info procs $p] == "$p"} then {$p} else {insert-debug "$arg"}
      }
   }
}

proc make-frame {} {
  global env actionlist module maxmodule modulelist maxcounter rates

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/vme_module" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE :layout T :padx 10 :pady 10
      :layout H; :row-gap 10

#      :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5 :value $module
#      :menubutton MODULE-SEL :label "select:" :strings $modulelist

#      :next-row T; :col-gap 20

      :message M1 :label "Counter Values"

      for {set i 0} {$i < $maxcounter} {incr i} {
        if {[expr $i/8*8] == $i} then {:next-row T}
        :text C[set i]DATA :label $i: :valuex 3c :w 10 :value 0 :deaf T
      }

      :next-row T
      :message M2 :label "Rate Values"

      for {set i 0} {$i < $maxcounter} {incr i} {
        if {[expr $i/8*8] == $i} then {:next-row T}
        :text C[set i]RATE :label $i: :valuex 3c :w 10 :value 0 :deaf T
      }

   :panel CLICK :layout T :padx 10
      :layout H; :col-gap 10

      :button REDISPLAY :label "Update Counters"  :command click-redisplay
      :button CLEAR     :label "Clear All Counters"
      :checkbox RATES    :label "Enable Rates" :value $rates

      set menu {}
      foreach item $actionlist {lappend menu [first $item]}
      :menubutton ACTIONS :label "More Actions"  :strings $menu

   :show T
}

#    list of menu items and associated actions for the ACTIONS menu button 

set actionlist {                         \
   {"Reset All Modules"           "click_reset"}     \
   {"Arm All Counters"            "click_arm"}       \
   {"Disarm All Counters"         "click_disarm"}     \
}

